#include "userfile.h"
#include "extensions.h"
#include <stdio.h>
#include <time.h>
#include <string.h>
#include <errno.h>
#include <unistd.h>
#include <sys/file.h>
#include <sys/types.h>
#include <sys/wait.h>
#include <sys/stat.h>
#include <dirent.h>
#include <signal.h>
#include <fcntl.h>
#include <ctype.h>

int euro = 0;
extern char	config_file[ BUFSIZ ];
extern char 	datapath[MAXPATHLEN];
extern char	rootpath[MAXPATHLEN];
extern int	section;
int load_userfile( char *username );
void update_user( struct UF user );
char *trim( char *str );

size_t commafmt( char *buf, int	bufsize, unsigned long N );

/***************************************************************************
  STRREV
  Reverses a string in place
***************************************************************************/

char *
strrev (char *str)
{
  char *p1, *p2;

  if (!str || !*str)
    return str;

  for (p1 = str, p2 = str + strlen (str) - 1; p2 > p1; ++p1, --p2)
    {
      *p1 ^= *p2;
      *p2 ^= *p1;
      *p1 ^= *p2;
    }
  return str;
}
/***************************************************************************
	COMMAFMT
	Puts commas in an unsigned long.
***************************************************************************/

size_t 
commafmt( char   		*buf,			/* Buffer for formatted string  */
          int     	bufsize, 	/* Size of buffer               */
          unsigned 	long N ) 	/* Number to convert            */
{
	int 	len = 1;
	int 	posn = 1;
	int 	sign = 1;
   char 	*ptr = buf + bufsize - 1;

	if ( 2 > bufsize )
   {
ABORT: 
		*buf = '\0';
		return 0;
   }

   *ptr-- = '\0';
   --bufsize;
	
   if ( 0L > N )
   {
   	sign = -1;
      N = -N;
   }

   for ( ; len <= bufsize; ++len, ++posn )
   {
   	*ptr-- = (char)( ( N % 10L ) + '0' );
   	if ( 0L == ( N /= 10L ) )
      	break;
   	if ( 0 == ( posn % 3 ) )
   	{
      	*ptr-- = ',';
         ++len;
   	}
   	if ( len >= bufsize )
         goto ABORT;
   }

   if ( 0 > sign )
   {
   	if ( len >= bufsize )
      	goto ABORT;
      *ptr-- = '-';
      ++len;
   }

   memmove( buf, ++ptr, len + 1 );
   return (size_t)len;
}
/*-- end of commafmt() ---------------------------------------------------*/

									 
/***************************************************************************
	TRIM
***************************************************************************/

char *
trim( char *str )
{
	char *ibuf;
	char *obuf;
												 
   if ( str )
   {
   	for ( ibuf = obuf = str; *ibuf; )
      {
        	while ( *ibuf && ( isspace ( *ibuf ) ) )
         	ibuf++;
        	if ( *ibuf && ( obuf != str ) )
            *( obuf++ ) = ' ';
        	while ( *ibuf && ( !isspace ( *ibuf ) ) )
            *( obuf++ ) = *( ibuf++ );
      }
      *obuf = '\0';
   }
   return( str );
}
/*-- end of trim() -------------------------------------------------------*/

/***************************************************************************
  CLEAR_UG_STRUCT
  Clears uf struct, setting default values where applicable.
***************************************************************************/

void 
clear_ug_struct( void )
{
   int ct;
												    
   memset( ug.name, '\0', sizeof( ug.name ) );
   memset( ug.flags, '\0', sizeof( ug.flags ) );
   ug.ratio = 3;
   ug.weekly_allotment = 0L;
   strncpy( ug.tagline, "No TagLine Set", sizeof(ug.tagline));
   ug.num_logins = 1;
   ug.idle_time = -1;
   ug.credits = 0L;
   ug.files_up = 0;
   ug.bytes_up = 0L;
   ug.seconds_up = 0L;
	
   ug.files_down = 0;
   ug.bytes_down = 0L;
   ug.seconds_down = 0L;
	
   ug.files_up_wk = 0;
   ug.bytes_up_wk = 0L;
   ug.seconds_up_wk = 0L;
	           
   ug.files_down_wk = 0;
   ug.bytes_down_wk = 0L;
   ug.seconds_down_wk = 0L;

   ug.files_up_day = 0;
   ug.bytes_up_day = 0L;
   ug.seconds_up_day = 0L;

   ug.files_down_day = 0;
   ug.bytes_down_day = 0L;
   ug.seconds_down_day = 0L;

   ug.files_up_month = 0;
   ug.bytes_up_month = 0L;
   ug.seconds_up_month = 0L;

   ug.files_down_month = 0;
   ug.bytes_down_month = 0L;
   ug.seconds_down_month = 0L;

   ug.login_times = 0;						  
   ug.last_on = (time_t)0;
   ug.time_limit = 0;
   ug.time_on_today = 0L;
   ug.last_nuked = (time_t)0;
   ug.times_nuked = 0;
   ug.bytes_nuked = 0L;
   ug.gadmin_max = -1;
   					 
   for ( ct = 0; ct < 10; ct++ ) {
      memset( ug.group_user[ ct ], '\0', sizeof( ug.group_user[ct]) );
      memset( ug.group_priv[ ct ], '\0', sizeof( ug.group_priv[ct]) );
      memset( ug.ip[ ct ], '\0', sizeof( ug.ip[ct]) );
   }
	
   return;
}
/*-- end of clear_uf_struct() --------------------------------------------*/
										  


/***************************************************************************
	PARSEVAL
	Parses values in user datafile.
***************************************************************************/

void 
userfile_parseval( struct USERRECORD lvalue )
{
   int ct;					

   if (!lvalue.arg[0])
   	return;
						
   if ( strcmp( lvalue.keyword, "GENERAL" ) == 0 )
   {
      ug.ratio = atoi( lvalue.arg[0]);
      if (lvalue.arg[1])
	      ug.weekly_allotment = strtoul(lvalue.arg[1], (char **)NULL, 0);
      if (lvalue.arg[2])
	      ug.idle_time = atoi(lvalue.arg[2]);
      if (lvalue.arg[3])
	      ug.credits = strtoul(lvalue.arg[3], (char **)NULL, 0);
      return;
   }  
	
   if ( strcmp( lvalue.keyword, "FLAGS" ) == 0 )
   {
      strncpy( ug.flags, lvalue.arg[0], sizeof(ug.flags));
      trim(ug.flags);
      return;
   }
	
   if ( strcmp( lvalue.keyword, "ALLUP" ) == 0 )
   {
      if (lvalue.arg[0+section])
	      ug.files_up = atoi( lvalue.arg[0+section]);
      if (lvalue.arg[1+section])
	      ug.bytes_up = strtoul(lvalue.arg[1+section], (char **)NULL, 0);
      if (lvalue.arg[2+section])
	      ug.seconds_up = strtoul(lvalue.arg[2+section], (char **)NULL, 0);
      return;
   }
	
   if ( strcmp( lvalue.keyword, "ALLDN" ) == 0 )
   {
      if (lvalue.arg[0+section])
      	      ug.files_down = atoi( lvalue.arg[0+section]);
      if (lvalue.arg[1+section])
	      ug.bytes_down = strtoul(lvalue.arg[1+section], (char **)NULL, 0);
      if (lvalue.arg[2+section])
	      ug.seconds_down = strtoul(lvalue.arg[2+section], (char **)NULL, 0);
      return;
   }
	
   if ( strcmp( lvalue.keyword, "WKUP" ) == 0 )
   {
      if (lvalue.arg[0+section])
              ug.files_up_wk = atoi( lvalue.arg[0+section]);
      if (lvalue.arg[1+section])
	      ug.bytes_up_wk = strtoul(lvalue.arg[1+section], (char **)NULL, 0);
      if (lvalue.arg[2+section])
	      ug.seconds_up_wk = strtoul(lvalue.arg[2+section], (char **)NULL, 0);	
      return;
   }
                           
   if ( strcmp( lvalue.keyword, "WKDN" ) == 0 )
   {
      if (lvalue.arg[0+section])
              ug.files_down_wk = atoi( lvalue.arg[0+section]);
      if (lvalue.arg[1+section])
	      ug.bytes_down_wk = strtoul(lvalue.arg[1+section], (char **)NULL, 0);
      if (lvalue.arg[2+section])
	      ug.seconds_down_wk = strtoul(lvalue.arg[2+section], (char **)NULL, 0);	
      return;
   }
 
   if ( strcmp( lvalue.keyword, "DAYUP" ) == 0 )
   {
      if (lvalue.arg[0+section])
              ug.files_up_day = atoi( lvalue.arg[0+section]);
      if (lvalue.arg[1+section])
	      ug.bytes_up_day = strtoul(lvalue.arg[1+section], (char **)NULL, 0);	
      if (lvalue.arg[2+section])
	      ug.seconds_up_day = strtoul(lvalue.arg[2+section], (char **)NULL, 0);	
      return;
   }

   if ( strcmp( lvalue.keyword, "DAYDN" ) == 0 )
   {
      if (lvalue.arg[0+section])
              ug.files_down_day = atoi( lvalue.arg[0+section] );
      if (lvalue.arg[1+section])
	      ug.bytes_down_day = strtoul(lvalue.arg[1+section], (char **)NULL, 0);	
      if (lvalue.arg[2+section])
      	      ug.seconds_down_day = strtoul(lvalue.arg[2+section], (char **)NULL, 0);
      return;
   }

   if ( strcmp( lvalue.keyword, "MONTHUP" ) == 0 )
   {
      if (lvalue.arg[0+section])
              ug.files_up_month = atoi( lvalue.arg[0+section] );
      if (lvalue.arg[1+section])
	      ug.bytes_up_month = strtoul(lvalue.arg[1+section], (char **)NULL, 0);
      if (lvalue.arg[2+section])
	      ug.seconds_up_month = strtoul(lvalue.arg[2+section], (char **)NULL, 0);	      	
      return;
   }

   if ( strcmp( lvalue.keyword, "MONTHDN" ) == 0 )
   {
      if (lvalue.arg[0+section])
              ug.files_down_month = atoi( lvalue.arg[0+section] );
      if (lvalue.arg[1+section])
	      ug.bytes_down_month = strtoul(lvalue.arg[1+section], (char **)NULL, 0);	
      if (lvalue.arg[2+section])
	      ug.seconds_down_month = strtoul(lvalue.arg[2+section], (char **)NULL, 0);
      return;
   }
   
   if ( strcmp( lvalue.keyword, "SLOTS" ) == 0 )
   {
     ug.gadmin_max = atoi(lvalue.arg[0]);
     return;
   }

   if ( strcmp( lvalue.keyword, "TIME" ) == 0 )
   {
      ug.login_times = atoi( lvalue.arg[0]);
      if (lvalue.arg[1])
	      ug.last_on = atol(lvalue.arg[1]);
      if (lvalue.arg[2])
	      ug.time_limit = atoi(lvalue.arg[2]);

      if ( ug.last_on )
      {
   		char datethen[30];
			char datenow[30];
			time_t timenow = time(NULL);
		
			sprintf( datethen, "%-24.24s", ctime(&ug.last_on) );
			sprintf( datenow, "%-24.24s", ctime(&timenow) );
			if ( !memcmp( datenow, datethen, 10 ) )
				ug.time_on_today = atol( lvalue.arg[3] );
			else
				ug.time_on_today = 0L;
      }
      else ug.time_on_today = 0L;	
      return;
   }                         
   
   if ( strcmp( lvalue.keyword, "NUKE" ) == 0 )
   {
      if (lvalue.arg[0+section])
              ug.last_nuked = atol( lvalue.arg[0+section] );
      if (lvalue.arg[1+section])
	      ug.times_nuked = atoi(lvalue.arg[1+section]);
      if (lvalue.arg[2+section])
	      ug.bytes_nuked = strtoul(lvalue.arg[2+section], (char **)NULL, 0);
      return;
   }						

   if ( strcmp( lvalue.keyword, "GROUP" ) == 0 )
	{
		for ( ct = 0; ct < 10; ct++ )
			if ( ug.group_user[ ct ][ 0 ] == '\0')
			{
				strcpy( ug.group_user[ ct ], lvalue.arg[0]);
            			trim( ug.group_user[ ct ] ); 
				return;
			}
	}
	 
   if ( strcmp( lvalue.keyword, "PRIVATE" ) == 0 )
	{
		for ( ct = 0; ct < 10; ct++ )
			if ( uf.group_priv[ ct ][ 0 ] == '\0')
			{
				strcpy( ug.group_priv[ ct ], lvalue.arg[0]);
			        trim( ug.group_priv[ ct ] ); 
				return;
			}
	}
	
   if ( strcmp( lvalue.keyword, "IP" ) == 0 )
	{
		for ( ct = 0; ct < 10; ct++ )
			if ( ug.ip[ ct ][ 0 ] == '\0')
			{
				strcpy( ug.ip[ ct ], lvalue.arg[0]);
		            	trim( ug.ip[ ct ] ); 
				return;
			}
	}
}  
/*-- end of userfile_parseval() -------------------------------------------*/



/***************************************************************************
	LOAD_USERFILE
	Loads user data for user "username".
	returns 1 on error, 0 otherwise.
***************************************************************************/

int
load_userfile( char *username )
{
	char *ptr, *aclptr;
	FILE *userfile;
	int cnt;
	char *aclbuf, *line;
	struct stat finfo;
	char fname_buff[MAXPATHLEN + 1];
	struct USERRECORD record;

	if (username != NULL) {
	clear_ug_struct();
		strncpy(ug.name, username, sizeof(ug.name)); 
	}

	sprintf(fname_buff, "%s/%s/users/%s",rootpath, datapath, username);
	if ((userfile = fopen(fname_buff, "r")) == NULL) {
		return 1;
	}
	if (fstat(fileno(userfile), &finfo) != 0) {
		fclose(userfile);
		return 1;
	}
	if (finfo.st_size == 0) {
		aclbuf = (char *)calloc(1,1);
	} else {
		if (!(aclbuf = (char *)malloc((unsigned) finfo.st_size + 1))) {
			(void)fclose(userfile);
			return 1;
		}
		if (!fread(aclbuf, (size_t)finfo.st_size, 1, userfile)) {
			aclbuf = NULL;
			(void)fclose(userfile);
			free(aclbuf);
			return 1;
		}
		*(aclbuf + finfo.st_size) = '\0';
	}
	(void)fclose(userfile);
	
	if (!aclbuf || !(*aclbuf)) {
		free(aclbuf);
		return 1;
	}
	aclptr = aclbuf;
	while (*aclptr != '\0') {
		line = aclptr;
		while (*aclptr && *aclptr != '\n')
			aclptr++;
		*aclptr++ = (char) NULL;
		
		if ((ptr = strchr(line, '#')) != NULL)
			*ptr = '\0';

		ptr = strtok(line, " \t");
		if (ptr) {
			if (strcmp(ptr, "UNFO") == 0) {
				strncpy(ug.tagline, line+5, sizeof(ug.tagline));
				trim(ug.tagline);
				continue;
			}
			if (strcmp(ptr, "TAGLINE") == 0) {
				strncpy(ug.tagline, line+8, sizeof(ug.tagline));
				trim(ug.tagline);
				continue;
			}
			strcpy(record.keyword, ptr);
		 	cnt = 0;
			while ((ptr = strtok(NULL, " \t")) != NULL) {
				if (cnt > MAX_ARGS)   /*MAX ARGS allowed in LINE */
					break;
				record.arg[cnt++] = ptr;
			}
			for (; cnt < MAX_ARGS; cnt++)
				record.arg[cnt] = NULL;
			userfile_parseval(record);
		}
	}
	free(aclbuf);
	uf = ug;
	return 0;
}
/*-- end of load_userfile() ----------------------------------------------*/
